"""LCM type definitions
This file automatically generated by lcm.
DO NOT MODIFY BY HAND!!!!
"""

try:
    import cStringIO.StringIO as BytesIO
except ImportError:
    from io import BytesIO
import struct

import compas_lcmtypes.stdlcm.header_t

class gamepad_t(object):
    __slots__ = ["header", "button_a", "button_b", "button_x", "button_y", "button_select", "button_start", "button_trigger_r", "button_trigger_l", "axis_r", "axis_l", "trigger_r", "trigger_l", "hats"]

    __typenames__ = ["compas_lcmtypes.stdlcm.header_t", "boolean", "boolean", "boolean", "boolean", "boolean", "boolean", "boolean", "boolean", "double", "double", "double", "double", "double"]

    __dimensions__ = [None, None, None, None, None, None, None, None, None, [2], [2], None, None, [2]]

    def __init__(self):
        self.header = compas_lcmtypes.stdlcm.header_t()
        self.button_a = False
        self.button_b = False
        self.button_x = False
        self.button_y = False
        self.button_select = False
        self.button_start = False
        self.button_trigger_r = False
        self.button_trigger_l = False
        self.axis_r = [ 0.0 for dim0 in range(2) ]
        self.axis_l = [ 0.0 for dim0 in range(2) ]
        self.trigger_r = 0.0
        self.trigger_l = 0.0
        self.hats = [ 0.0 for dim0 in range(2) ]

    def encode(self):
        buf = BytesIO()
        buf.write(gamepad_t._get_packed_fingerprint())
        self._encode_one(buf)
        return buf.getvalue()

    def _encode_one(self, buf):
        assert self.header._get_packed_fingerprint() == compas_lcmtypes.stdlcm.header_t._get_packed_fingerprint()
        self.header._encode_one(buf)
        buf.write(struct.pack(">bbbbbbbb", self.button_a, self.button_b, self.button_x, self.button_y, self.button_select, self.button_start, self.button_trigger_r, self.button_trigger_l))
        buf.write(struct.pack('>2d', *self.axis_r[:2]))
        buf.write(struct.pack('>2d', *self.axis_l[:2]))
        buf.write(struct.pack(">dd", self.trigger_r, self.trigger_l))
        buf.write(struct.pack('>2d', *self.hats[:2]))

    def decode(data):
        if hasattr(data, 'read'):
            buf = data
        else:
            buf = BytesIO(data)
        if buf.read(8) != gamepad_t._get_packed_fingerprint():
            raise ValueError("Decode error")
        return gamepad_t._decode_one(buf)
    decode = staticmethod(decode)

    def _decode_one(buf):
        self = gamepad_t()
        self.header = compas_lcmtypes.stdlcm.header_t._decode_one(buf)
        self.button_a = bool(struct.unpack('b', buf.read(1))[0])
        self.button_b = bool(struct.unpack('b', buf.read(1))[0])
        self.button_x = bool(struct.unpack('b', buf.read(1))[0])
        self.button_y = bool(struct.unpack('b', buf.read(1))[0])
        self.button_select = bool(struct.unpack('b', buf.read(1))[0])
        self.button_start = bool(struct.unpack('b', buf.read(1))[0])
        self.button_trigger_r = bool(struct.unpack('b', buf.read(1))[0])
        self.button_trigger_l = bool(struct.unpack('b', buf.read(1))[0])
        self.axis_r = struct.unpack('>2d', buf.read(16))
        self.axis_l = struct.unpack('>2d', buf.read(16))
        self.trigger_r, self.trigger_l = struct.unpack(">dd", buf.read(16))
        self.hats = struct.unpack('>2d', buf.read(16))
        return self
    _decode_one = staticmethod(_decode_one)

    def _get_hash_recursive(parents):
        if gamepad_t in parents: return 0
        newparents = parents + [gamepad_t]
        tmphash = (0xa2323731c07047db+ compas_lcmtypes.stdlcm.header_t._get_hash_recursive(newparents)) & 0xffffffffffffffff
        tmphash  = (((tmphash<<1)&0xffffffffffffffff) + (tmphash>>63)) & 0xffffffffffffffff
        return tmphash
    _get_hash_recursive = staticmethod(_get_hash_recursive)
    _packed_fingerprint = None

    def _get_packed_fingerprint():
        if gamepad_t._packed_fingerprint is None:
            gamepad_t._packed_fingerprint = struct.pack(">Q", gamepad_t._get_hash_recursive([]))
        return gamepad_t._packed_fingerprint
    _get_packed_fingerprint = staticmethod(_get_packed_fingerprint)

    def get_hash(self):
        """Get the LCM hash of the struct"""
        return struct.unpack(">Q", gamepad_t._get_packed_fingerprint())[0]




"""LCM package __init__.py file
This file automatically generated by lcm-gen.
DO NOT MODIFY BY HAND!!!!
"""

from .odometry_t import odometry_t
from .pose_t import pose_t
from .position_t import position_t
from .twist_t import twist_t



"""LCM type definitions
This file automatically generated by lcm.
DO NOT MODIFY BY HAND!!!!
"""

try:
    import cStringIO.StringIO as BytesIO
except ImportError:
    from io import BytesIO
import struct

import compas_lcmtypes.stdlcm.string_entry_t

import compas_lcmtypes.stdlcm.header_t

import compas_lcmtypes.stdlcm.bool_entry_t

import compas_lcmtypes.stdlcm.real_entry_t

import compas_lcmtypes.stdlcm.byte_array_entry_t

import compas_lcmtypes.stdlcm.integer_entry_t

class msg_t(object):
    __slots__ = ["header", "n_bools", "l_bool", "n_ints", "l_int", "n_reals", "l_real", "n_strings", "l_string", "n_bytes", "l_bytes"]

    __typenames__ = ["compas_lcmtypes.stdlcm.header_t", "int16_t", "compas_lcmtypes.stdlcm.bool_entry_t", "int16_t", "compas_lcmtypes.stdlcm.integer_entry_t", "int16_t", "compas_lcmtypes.stdlcm.real_entry_t", "int16_t", "compas_lcmtypes.stdlcm.string_entry_t", "int16_t", "compas_lcmtypes.stdlcm.byte_array_entry_t"]

    __dimensions__ = [None, None, ["n_bools"], None, ["n_ints"], None, ["n_reals"], None, ["n_strings"], None, ["n_bytes"]]

    def __init__(self):
        self.header = compas_lcmtypes.stdlcm.header_t()
        self.n_bools = 0
        self.l_bool = []
        self.n_ints = 0
        self.l_int = []
        self.n_reals = 0
        self.l_real = []
        self.n_strings = 0
        self.l_string = []
        self.n_bytes = 0
        self.l_bytes = []

    def encode(self):
        buf = BytesIO()
        buf.write(msg_t._get_packed_fingerprint())
        self._encode_one(buf)
        return buf.getvalue()

    def _encode_one(self, buf):
        assert self.header._get_packed_fingerprint() == compas_lcmtypes.stdlcm.header_t._get_packed_fingerprint()
        self.header._encode_one(buf)
        buf.write(struct.pack(">h", self.n_bools))
        for i0 in range(self.n_bools):
            assert self.l_bool[i0]._get_packed_fingerprint() == compas_lcmtypes.stdlcm.bool_entry_t._get_packed_fingerprint()
            self.l_bool[i0]._encode_one(buf)
        buf.write(struct.pack(">h", self.n_ints))
        for i0 in range(self.n_ints):
            assert self.l_int[i0]._get_packed_fingerprint() == compas_lcmtypes.stdlcm.integer_entry_t._get_packed_fingerprint()
            self.l_int[i0]._encode_one(buf)
        buf.write(struct.pack(">h", self.n_reals))
        for i0 in range(self.n_reals):
            assert self.l_real[i0]._get_packed_fingerprint() == compas_lcmtypes.stdlcm.real_entry_t._get_packed_fingerprint()
            self.l_real[i0]._encode_one(buf)
        buf.write(struct.pack(">h", self.n_strings))
        for i0 in range(self.n_strings):
            assert self.l_string[i0]._get_packed_fingerprint() == compas_lcmtypes.stdlcm.string_entry_t._get_packed_fingerprint()
            self.l_string[i0]._encode_one(buf)
        buf.write(struct.pack(">h", self.n_bytes))
        for i0 in range(self.n_bytes):
            assert self.l_bytes[i0]._get_packed_fingerprint() == compas_lcmtypes.stdlcm.byte_array_entry_t._get_packed_fingerprint()
            self.l_bytes[i0]._encode_one(buf)

    def decode(data):
        if hasattr(data, 'read'):
            buf = data
        else:
            buf = BytesIO(data)
        if buf.read(8) != msg_t._get_packed_fingerprint():
            raise ValueError("Decode error")
        return msg_t._decode_one(buf)
    decode = staticmethod(decode)

    def _decode_one(buf):
        self = msg_t()
        self.header = compas_lcmtypes.stdlcm.header_t._decode_one(buf)
        self.n_bools = struct.unpack(">h", buf.read(2))[0]
        self.l_bool = []
        for i0 in range(self.n_bools):
            self.l_bool.append(compas_lcmtypes.stdlcm.bool_entry_t._decode_one(buf))
        self.n_ints = struct.unpack(">h", buf.read(2))[0]
        self.l_int = []
        for i0 in range(self.n_ints):
            self.l_int.append(compas_lcmtypes.stdlcm.integer_entry_t._decode_one(buf))
        self.n_reals = struct.unpack(">h", buf.read(2))[0]
        self.l_real = []
        for i0 in range(self.n_reals):
            self.l_real.append(compas_lcmtypes.stdlcm.real_entry_t._decode_one(buf))
        self.n_strings = struct.unpack(">h", buf.read(2))[0]
        self.l_string = []
        for i0 in range(self.n_strings):
            self.l_string.append(compas_lcmtypes.stdlcm.string_entry_t._decode_one(buf))
        self.n_bytes = struct.unpack(">h", buf.read(2))[0]
        self.l_bytes = []
        for i0 in range(self.n_bytes):
            self.l_bytes.append(compas_lcmtypes.stdlcm.byte_array_entry_t._decode_one(buf))
        return self
    _decode_one = staticmethod(_decode_one)

    def _get_hash_recursive(parents):
        if msg_t in parents: return 0
        newparents = parents + [msg_t]
        tmphash = (0x4c8a8b3617ce65f5+ compas_lcmtypes.stdlcm.header_t._get_hash_recursive(newparents)+ compas_lcmtypes.stdlcm.bool_entry_t._get_hash_recursive(newparents)+ compas_lcmtypes.stdlcm.integer_entry_t._get_hash_recursive(newparents)+ compas_lcmtypes.stdlcm.real_entry_t._get_hash_recursive(newparents)+ compas_lcmtypes.stdlcm.string_entry_t._get_hash_recursive(newparents)+ compas_lcmtypes.stdlcm.byte_array_entry_t._get_hash_recursive(newparents)) & 0xffffffffffffffff
        tmphash  = (((tmphash<<1)&0xffffffffffffffff) + (tmphash>>63)) & 0xffffffffffffffff
        return tmphash
    _get_hash_recursive = staticmethod(_get_hash_recursive)
    _packed_fingerprint = None

    def _get_packed_fingerprint():
        if msg_t._packed_fingerprint is None:
            msg_t._packed_fingerprint = struct.pack(">Q", msg_t._get_hash_recursive([]))
        return msg_t._packed_fingerprint
    _get_packed_fingerprint = staticmethod(_get_packed_fingerprint)

    def get_hash(self):
        """Get the LCM hash of the struct"""
        return struct.unpack(">Q", msg_t._get_packed_fingerprint())[0]




"""LCM type definitions
This file automatically generated by lcm.
DO NOT MODIFY BY HAND!!!!
"""

try:
    import cStringIO.StringIO as BytesIO
except ImportError:
    from io import BytesIO
import struct

import compas_lcmtypes.stdlcm.header_t

class battery_t(object):
    __slots__ = ["header", "voltage", "temperature", "current", "charge"]

    __typenames__ = ["compas_lcmtypes.stdlcm.header_t", "double", "double", "double", "double"]

    __dimensions__ = [None, None, None, None, None]

    def __init__(self):
        self.header = compas_lcmtypes.stdlcm.header_t()
        self.voltage = 0.0
        self.temperature = 0.0
        self.current = 0.0
        self.charge = 0.0

    def encode(self):
        buf = BytesIO()
        buf.write(battery_t._get_packed_fingerprint())
        self._encode_one(buf)
        return buf.getvalue()

    def _encode_one(self, buf):
        assert self.header._get_packed_fingerprint() == compas_lcmtypes.stdlcm.header_t._get_packed_fingerprint()
        self.header._encode_one(buf)
        buf.write(struct.pack(">dddd", self.voltage, self.temperature, self.current, self.charge))

    def decode(data):
        if hasattr(data, 'read'):
            buf = data
        else:
            buf = BytesIO(data)
        if buf.read(8) != battery_t._get_packed_fingerprint():
            raise ValueError("Decode error")
        return battery_t._decode_one(buf)
    decode = staticmethod(decode)

    def _decode_one(buf):
        self = battery_t()
        self.header = compas_lcmtypes.stdlcm.header_t._decode_one(buf)
        self.voltage, self.temperature, self.current, self.charge = struct.unpack(">dddd", buf.read(32))
        return self
    _decode_one = staticmethod(_decode_one)

    def _get_hash_recursive(parents):
        if battery_t in parents: return 0
        newparents = parents + [battery_t]
        tmphash = (0x8ebb4d983630afcf+ compas_lcmtypes.stdlcm.header_t._get_hash_recursive(newparents)) & 0xffffffffffffffff
        tmphash  = (((tmphash<<1)&0xffffffffffffffff) + (tmphash>>63)) & 0xffffffffffffffff
        return tmphash
    _get_hash_recursive = staticmethod(_get_hash_recursive)
    _packed_fingerprint = None

    def _get_packed_fingerprint():
        if battery_t._packed_fingerprint is None:
            battery_t._packed_fingerprint = struct.pack(">Q", battery_t._get_hash_recursive([]))
        return battery_t._packed_fingerprint
    _get_packed_fingerprint = staticmethod(_get_packed_fingerprint)

    def get_hash(self):
        """Get the LCM hash of the struct"""
        return struct.unpack(">Q", battery_t._get_packed_fingerprint())[0]




"""LCM package __init__.py file
This file automatically generated by lcm-gen.
DO NOT MODIFY BY HAND!!!!
"""

from .ahrs_t import ahrs_t
from .battery_t import battery_t
from .camera_info import camera_info
from .cmd_effort_t import cmd_effort_t
from .cmd_list_t import cmd_list_t
from .ctd_t import ctd_t
from .depth_t import depth_t
from .dvl_t import dvl_t
from .gamepad_t import gamepad_t
from .gps_fix_t import gps_fix_t
from .image_metadata_t import image_metadata_t
from .image_sync_t import image_sync_t
from .image_t import image_t
from .imu_t import imu_t
from .multibeam_t import multibeam_t
from .rph_angrate_t import rph_angrate_t
from .rph_t import rph_t



"""LCM type definitions
This file automatically generated by lcm.
DO NOT MODIFY BY HAND!!!!
"""

try:
    import cStringIO.StringIO as BytesIO
except ImportError:
    from io import BytesIO
import struct

import compas_lcmtypes.stdlcm.header_t

class ahrs_t(object):
    __slots__ = ["header", "orientation", "angRate", "xyz_ddot", "magfield"]

    __typenames__ = ["compas_lcmtypes.stdlcm.header_t", "double", "double", "double", "double"]

    __dimensions__ = [None, [4], [3], [3], [3]]

    def __init__(self):
        self.header = compas_lcmtypes.stdlcm.header_t()
        self.orientation = [ 0.0 for dim0 in range(4) ]
        self.angRate = [ 0.0 for dim0 in range(3) ]
        self.xyz_ddot = [ 0.0 for dim0 in range(3) ]
        self.magfield = [ 0.0 for dim0 in range(3) ]

    def encode(self):
        buf = BytesIO()
        buf.write(ahrs_t._get_packed_fingerprint())
        self._encode_one(buf)
        return buf.getvalue()

    def _encode_one(self, buf):
        assert self.header._get_packed_fingerprint() == compas_lcmtypes.stdlcm.header_t._get_packed_fingerprint()
        self.header._encode_one(buf)
        buf.write(struct.pack('>4d', *self.orientation[:4]))
        buf.write(struct.pack('>3d', *self.angRate[:3]))
        buf.write(struct.pack('>3d', *self.xyz_ddot[:3]))
        buf.write(struct.pack('>3d', *self.magfield[:3]))

    def decode(data):
        if hasattr(data, 'read'):
            buf = data
        else:
            buf = BytesIO(data)
        if buf.read(8) != ahrs_t._get_packed_fingerprint():
            raise ValueError("Decode error")
        return ahrs_t._decode_one(buf)
    decode = staticmethod(decode)

    def _decode_one(buf):
        self = ahrs_t()
        self.header = compas_lcmtypes.stdlcm.header_t._decode_one(buf)
        self.orientation = struct.unpack('>4d', buf.read(32))
        self.angRate = struct.unpack('>3d', buf.read(24))
        self.xyz_ddot = struct.unpack('>3d', buf.read(24))
        self.magfield = struct.unpack('>3d', buf.read(24))
        return self
    _decode_one = staticmethod(_decode_one)

    def _get_hash_recursive(parents):
        if ahrs_t in parents: return 0
        newparents = parents + [ahrs_t]
        tmphash = (0x7ffb7870ea25455+ compas_lcmtypes.stdlcm.header_t._get_hash_recursive(newparents)) & 0xffffffffffffffff
        tmphash  = (((tmphash<<1)&0xffffffffffffffff) + (tmphash>>63)) & 0xffffffffffffffff
        return tmphash
    _get_hash_recursive = staticmethod(_get_hash_recursive)
    _packed_fingerprint = None

    def _get_packed_fingerprint():
        if ahrs_t._packed_fingerprint is None:
            ahrs_t._packed_fingerprint = struct.pack(">Q", ahrs_t._get_hash_recursive([]))
        return ahrs_t._packed_fingerprint
    _get_packed_fingerprint = staticmethod(_get_packed_fingerprint)

    def get_hash(self):
        """Get the LCM hash of the struct"""
        return struct.unpack(">Q", ahrs_t._get_packed_fingerprint())[0]




"""LCM type definitions
This file automatically generated by lcm.
DO NOT MODIFY BY HAND!!!!
"""

try:
    import cStringIO.StringIO as BytesIO
except ImportError:
    from io import BytesIO
import struct

import compas_lcmtypes.stdlcm.header_t

import compas_lcmtypes.senlcm.image_metadata_t

class image_t(object):
    __slots__ = ["header", "width", "height", "row_stride", "pixelformat", "size", "data", "nmetadata", "metadata"]

    __typenames__ = ["compas_lcmtypes.stdlcm.header_t", "int32_t", "int32_t", "int32_t", "int32_t", "int32_t", "byte", "int32_t", "compas_lcmtypes.senlcm.image_metadata_t"]

    __dimensions__ = [None, None, None, None, None, None, ["size"], None, ["nmetadata"]]

    PIXEL_FORMAT_UYVY = 1498831189
    PIXEL_FORMAT_YUYV = 1448695129
    PIXEL_FORMAT_IYU1 = 827677001
    PIXEL_FORMAT_IYU2 = 844454217
    PIXEL_FORMAT_YUV420 = 842093913
    PIXEL_FORMAT_YUV411P = 1345401140
    PIXEL_FORMAT_I420 = 808596553
    PIXEL_FORMAT_NV12 = 842094158
    PIXEL_FORMAT_GRAY = 1497715271
    PIXEL_FORMAT_RGB = 859981650
    PIXEL_FORMAT_BGR = 861030210
    PIXEL_FORMAT_RGBA = 876758866
    PIXEL_FORMAT_BGRA = 877807426
    PIXEL_FORMAT_BAYER_BGGR = 825770306
    PIXEL_FORMAT_BAYER_GBRG = 844650584
    PIXEL_FORMAT_BAYER_GRBG = 861427800
    PIXEL_FORMAT_BAYER_RGGB = 878205016
    PIXEL_FORMAT_BE_BAYER16_BGGR = 826360386
    PIXEL_FORMAT_BE_BAYER16_GBRG = 843137602
    PIXEL_FORMAT_BE_BAYER16_GRBG = 859914818
    PIXEL_FORMAT_BE_BAYER16_RGGB = 876692034
    PIXEL_FORMAT_LE_BAYER16_BGGR = 826360396
    PIXEL_FORMAT_LE_BAYER16_GBRG = 843137612
    PIXEL_FORMAT_LE_BAYER16_GRBG = 859914828
    PIXEL_FORMAT_LE_BAYER16_RGGB = 876692044
    PIXEL_FORMAT_MJPEG = 1196444237
    PIXEL_FORMAT_BE_GRAY16 = 357
    PIXEL_FORMAT_LE_GRAY16 = 909199180
    PIXEL_FORMAT_BE_RGB16 = 358
    PIXEL_FORMAT_LE_RGB16 = 1279412050
    PIXEL_FORMAT_BE_SIGNED_GRAY16 = 359
    PIXEL_FORMAT_BE_SIGNED_RGB16 = 360
    PIXEL_FORMAT_FLOAT_GRAY32 = 842221382
    PIXEL_FORMAT_INVALID = -2
    PIXEL_FORMAT_ANY = -1

    def __init__(self):
        self.header = compas_lcmtypes.stdlcm.header_t()
        self.width = 0
        self.height = 0
        self.row_stride = 0
        self.pixelformat = 0
        self.size = 0
        self.data = b""
        self.nmetadata = 0
        self.metadata = []

    def encode(self):
        buf = BytesIO()
        buf.write(image_t._get_packed_fingerprint())
        self._encode_one(buf)
        return buf.getvalue()

    def _encode_one(self, buf):
        assert self.header._get_packed_fingerprint() == compas_lcmtypes.stdlcm.header_t._get_packed_fingerprint()
        self.header._encode_one(buf)
        buf.write(struct.pack(">iiiii", self.width, self.height, self.row_stride, self.pixelformat, self.size))
        buf.write(bytearray(self.data[:self.size]))
        buf.write(struct.pack(">i", self.nmetadata))
        for i0 in range(self.nmetadata):
            assert self.metadata[i0]._get_packed_fingerprint() == compas_lcmtypes.senlcm.image_metadata_t._get_packed_fingerprint()
            self.metadata[i0]._encode_one(buf)

    def decode(data):
        if hasattr(data, 'read'):
            buf = data
        else:
            buf = BytesIO(data)
        if buf.read(8) != image_t._get_packed_fingerprint():
            raise ValueError("Decode error")
        return image_t._decode_one(buf)
    decode = staticmethod(decode)

    def _decode_one(buf):
        self = image_t()
        self.header = compas_lcmtypes.stdlcm.header_t._decode_one(buf)
        self.width, self.height, self.row_stride, self.pixelformat, self.size = struct.unpack(">iiiii", buf.read(20))
        self.data = buf.read(self.size)
        self.nmetadata = struct.unpack(">i", buf.read(4))[0]
        self.metadata = []
        for i0 in range(self.nmetadata):
            self.metadata.append(compas_lcmtypes.senlcm.image_metadata_t._decode_one(buf))
        return self
    _decode_one = staticmethod(_decode_one)

    def _get_hash_recursive(parents):
        if image_t in parents: return 0
        newparents = parents + [image_t]
        tmphash = (0x5f0325a1392627aa+ compas_lcmtypes.stdlcm.header_t._get_hash_recursive(newparents)+ compas_lcmtypes.senlcm.image_metadata_t._get_hash_recursive(newparents)) & 0xffffffffffffffff
        tmphash  = (((tmphash<<1)&0xffffffffffffffff) + (tmphash>>63)) & 0xffffffffffffffff
        return tmphash
    _get_hash_recursive = staticmethod(_get_hash_recursive)
    _packed_fingerprint = None

    def _get_packed_fingerprint():
        if image_t._packed_fingerprint is None:
            image_t._packed_fingerprint = struct.pack(">Q", image_t._get_hash_recursive([]))
        return image_t._packed_fingerprint
    _get_packed_fingerprint = staticmethod(_get_packed_fingerprint)

    def get_hash(self):
        """Get the LCM hash of the struct"""
        return struct.unpack(">Q", image_t._get_packed_fingerprint())[0]




"""LCM package __init__.py file
This file automatically generated by lcm-gen.
DO NOT MODIFY BY HAND!!!!
"""

from .point_t import point_t
from .pose_t import pose_t
from .pose_with_covariance_t import pose_with_covariance_t
from .quaternion_t import quaternion_t
from .transform_t import transform_t
from .twist_t import twist_t
from .twist_with_covariance_t import twist_with_covariance_t
from .vector3_t import vector3_t



"""LCM type definitions
This file automatically generated by lcm.
DO NOT MODIFY BY HAND!!!!
"""

try:
    import cStringIO.StringIO as BytesIO
except ImportError:
    from io import BytesIO
import struct

class integer_entry_t(object):
    __slots__ = ["key", "value"]

    __typenames__ = ["string", "int64_t"]

    __dimensions__ = [None, None]

    def __init__(self):
        self.key = ""
        self.value = 0

    def encode(self):
        buf = BytesIO()
        buf.write(integer_entry_t._get_packed_fingerprint())
        self._encode_one(buf)
        return buf.getvalue()

    def _encode_one(self, buf):
        __key_encoded = self.key.encode('utf-8')
        buf.write(struct.pack('>I', len(__key_encoded)+1))
        buf.write(__key_encoded)
        buf.write(b"\0")
        buf.write(struct.pack(">q", self.value))

    def decode(data):
        if hasattr(data, 'read'):
            buf = data
        else:
            buf = BytesIO(data)
        if buf.read(8) != integer_entry_t._get_packed_fingerprint():
            raise ValueError("Decode error")
        return integer_entry_t._decode_one(buf)
    decode = staticmethod(decode)

    def _decode_one(buf):
        self = integer_entry_t()
        __key_len = struct.unpack('>I', buf.read(4))[0]
        self.key = buf.read(__key_len)[:-1].decode('utf-8', 'replace')
        self.value = struct.unpack(">q", buf.read(8))[0]
        return self
    _decode_one = staticmethod(_decode_one)

    def _get_hash_recursive(parents):
        if integer_entry_t in parents: return 0
        tmphash = (0xb2b9e7f79709ad2c) & 0xffffffffffffffff
        tmphash  = (((tmphash<<1)&0xffffffffffffffff) + (tmphash>>63)) & 0xffffffffffffffff
        return tmphash
    _get_hash_recursive = staticmethod(_get_hash_recursive)
    _packed_fingerprint = None

    def _get_packed_fingerprint():
        if integer_entry_t._packed_fingerprint is None:
            integer_entry_t._packed_fingerprint = struct.pack(">Q", integer_entry_t._get_hash_recursive([]))
        return integer_entry_t._packed_fingerprint
    _get_packed_fingerprint = staticmethod(_get_packed_fingerprint)

    def get_hash(self):
        """Get the LCM hash of the struct"""
        return struct.unpack(">Q", integer_entry_t._get_packed_fingerprint())[0]




"""LCM type definitions
This file automatically generated by lcm.
DO NOT MODIFY BY HAND!!!!
"""

try:
    import cStringIO.StringIO as BytesIO
except ImportError:
    from io import BytesIO
import struct

import compas_lcmtypes.stdlcm.header_t

class camera_info(object):
    __slots__ = ["header", "height", "width", "distortion_model", "dsize", "D", "K", "R", "P", "binning_x", "binning_y", "roi", "roi_do_rectify"]

    __typenames__ = ["compas_lcmtypes.stdlcm.header_t", "int32_t", "int32_t", "string", "int32_t", "double", "double", "double", "double", "int32_t", "int32_t", "int32_t", "boolean"]

    __dimensions__ = [None, None, None, None, None, ["dsize"], [9], [9], [12], None, None, [4], None]

    def __init__(self):
        self.header = compas_lcmtypes.stdlcm.header_t()
        self.height = 0
        self.width = 0
        self.distortion_model = ""
        self.dsize = 0
        self.D = []
        self.K = [ 0.0 for dim0 in range(9) ]
        self.R = [ 0.0 for dim0 in range(9) ]
        self.P = [ 0.0 for dim0 in range(12) ]
        self.binning_x = 0
        self.binning_y = 0
        self.roi = [ 0 for dim0 in range(4) ]
        self.roi_do_rectify = False

    def encode(self):
        buf = BytesIO()
        buf.write(camera_info._get_packed_fingerprint())
        self._encode_one(buf)
        return buf.getvalue()

    def _encode_one(self, buf):
        assert self.header._get_packed_fingerprint() == compas_lcmtypes.stdlcm.header_t._get_packed_fingerprint()
        self.header._encode_one(buf)
        buf.write(struct.pack(">ii", self.height, self.width))
        __distortion_model_encoded = self.distortion_model.encode('utf-8')
        buf.write(struct.pack('>I', len(__distortion_model_encoded)+1))
        buf.write(__distortion_model_encoded)
        buf.write(b"\0")
        buf.write(struct.pack(">i", self.dsize))
        buf.write(struct.pack('>%dd' % self.dsize, *self.D[:self.dsize]))
        buf.write(struct.pack('>9d', *self.K[:9]))
        buf.write(struct.pack('>9d', *self.R[:9]))
        buf.write(struct.pack('>12d', *self.P[:12]))
        buf.write(struct.pack(">ii", self.binning_x, self.binning_y))
        buf.write(struct.pack('>4i', *self.roi[:4]))
        buf.write(struct.pack(">b", self.roi_do_rectify))

    def decode(data):
        if hasattr(data, 'read'):
            buf = data
        else:
            buf = BytesIO(data)
        if buf.read(8) != camera_info._get_packed_fingerprint():
            raise ValueError("Decode error")
        return camera_info._decode_one(buf)
    decode = staticmethod(decode)

    def _decode_one(buf):
        self = camera_info()
        self.header = compas_lcmtypes.stdlcm.header_t._decode_one(buf)
        self.height, self.width = struct.unpack(">ii", buf.read(8))
        __distortion_model_len = struct.unpack('>I', buf.read(4))[0]
        self.distortion_model = buf.read(__distortion_model_len)[:-1].decode('utf-8', 'replace')
        self.dsize = struct.unpack(">i", buf.read(4))[0]
        self.D = struct.unpack('>%dd' % self.dsize, buf.read(self.dsize * 8))
        self.K = struct.unpack('>9d', buf.read(72))
        self.R = struct.unpack('>9d', buf.read(72))
        self.P = struct.unpack('>12d', buf.read(96))
        self.binning_x, self.binning_y = struct.unpack(">ii", buf.read(8))
        self.roi = struct.unpack('>4i', buf.read(16))
        self.roi_do_rectify = bool(struct.unpack('b', buf.read(1))[0])
        return self
    _decode_one = staticmethod(_decode_one)

    def _get_hash_recursive(parents):
        if camera_info in parents: return 0
        newparents = parents + [camera_info]
        tmphash = (0x7edeeeafed8d83c0+ compas_lcmtypes.stdlcm.header_t._get_hash_recursive(newparents)) & 0xffffffffffffffff
        tmphash  = (((tmphash<<1)&0xffffffffffffffff) + (tmphash>>63)) & 0xffffffffffffffff
        return tmphash
    _get_hash_recursive = staticmethod(_get_hash_recursive)
    _packed_fingerprint = None

    def _get_packed_fingerprint():
        if camera_info._packed_fingerprint is None:
            camera_info._packed_fingerprint = struct.pack(">Q", camera_info._get_hash_recursive([]))
        return camera_info._packed_fingerprint
    _get_packed_fingerprint = staticmethod(_get_packed_fingerprint)

    def get_hash(self):
        """Get the LCM hash of the struct"""
        return struct.unpack(">Q", camera_info._get_packed_fingerprint())[0]




"""LCM package __init__.py file
This file automatically generated by lcm-gen.
DO NOT MODIFY BY HAND!!!!
"""

from .header_t import header_t
from .bool_entry_t import bool_entry_t
from .integer_entry_t import integer_entry_t
from .real_entry_t import real_entry_t
from .string_entry_t import string_entry_t
from .byte_array_entry_t import byte_array_entry_t
from .msg_t import msg_t
from .raw_t import raw_t
